package saccubus.net;

import static org.apache.commons.lang.StringUtils.*;

import java.io.File;
import java.io.IOException;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author yuki
 */
public class CookieWinCrome extends Cookie {

    private static final Logger logger = LoggerFactory.getLogger(CookieWinCrome.class);
    private static final String COOKIE_PATH = "\\Google\\Chrome\\User Data\\Default\\Cookies";

    /** <p>
     *  Chrome から user_session を取得。エラーが起こった場合、例外を投げずに空文字を返す
     *  </p>
     *  @return user_session
     */
    @Override
    public String getUserSessionString() throws IOException {
        final String cookieWin7 = tryReadingWin7();
        if (isNotEmpty(cookieWin7)) {
            return cookieWin7;
        }

        final String cookieXp = tryReadingXp();
        if (isNotEmpty(cookieXp)) {
            return cookieXp;
        }

        final String cookieOther = tryReadingOther();
        if (isNotEmpty(cookieOther)) {
            return cookieOther;
        }

        throw new IOException("user session not found in google chorome cookie");
    }

    /**
     * Windows7 のcookie取得を試みる.
     * @return ユーザセッション文字列. 取得できない場合はnull.
     */
    private String tryReadingWin7() {
        final String localAppData = System.getenv("LOCALAPPDATA");
        return tryReading(localAppData);
    }

    /**
     * WindowsXP のcookie処理を試みる.
     * @return ユーザセッション文字列. 取得できない場合はnull.
     */
    private String tryReadingXp() {
        final String userProfile = System.getenv("USERPROFILE");
        if (isNotEmpty(userProfile)) {
            // XP 32bit
            return tryReading(userProfile + "\\Local Settings\\Application Data");
        }
        return null;
    }

    private String tryReadingOther() {
        // ??? just try
        final String appData = System.getenv("APPDATA");
        return tryReading(appData);
    }

    private String tryReading(String pathPrefix) {
        if (isNotEmpty(pathPrefix)) {
            final File cookieFile = new File(pathPrefix + COOKIE_PATH);
            if (cookieFile.isFile()) {
                try {
                    String dataStr = FileUtils.readFileToString(cookieFile, "UTF-8");
                    return cutUserSession(dataStr);
                } catch (IOException ex) {
                    logger.error("cookie read error: " + cookieFile.getPath(), ex);
                }
            }
        }
        return null;
    }
}
